// Require the necessary discord.js classes
const fs = require('node:fs');
const path = require('node:path');
const { Client, Collection, Events, GatewayIntentBits } = require('discord.js');
const { token } = require('./config.json');

// Create a new client instance
const client = new Client({ intents: [GatewayIntentBits.Guilds] });

// Handle dynamically reading and executing commands.
client.commands = new Collection();

// Retrieve the commands directory and the files inside of it, including subdirectories.
const foldersPath = path.join(__dirname, 'commands');
const commandDirectories = fs.readdirSync(foldersPath);

// Load the command files.
for (const directory of commandDirectories) {
    const commandsPath = path.join(foldersPath, directory);
    const commandFiles = fs.readdirSync(commandsPath).filter(file => file.endsWith('.js'));
    for (const file of commandFiles) {
        const filePath = path.join(commandsPath, file);
        const command = require(filePath);
        // Set a new item in the Collection with the key as the command's name and the value as the exported module.
        if ('data' in command && 'execute' in command) {
            client.commands.set(command.data.name, command);
        } else {
            console.log(`[WARNING] The command at ${filePath} is missing a required "data" or "execute" property.`);
        }
    }
}

// When the client is ready, run this code (only once)
// We use 'c' for the event parameter to keep it separate from the already defined 'client'
client.once(Events.ClientReady, c => {
	console.log(`Ready! Logged in as ${c.user.tag}`);
});

// Create a listener to respond to interaction events.
client.on(Events.InteractionCreate, async interaction => {
    if (!interaction.isChatInputCommand()) return;

    const command = interaction.client.commands.get(interaction.commandName);

    if (!command) {
        console.error(`No command matching ${interaction.commandName} was found.`);
        return;
    }

    try {
        await command.execute(interaction);
    } catch (error) {
    console.error(error);
    if (interaction.replied || interaction.deffered) {
        await interaction.followUp({ content: 'There was an error while executing this command!', epheremal: true });
    } else {
        await interaction.reply({ content: 'There was an error while executing this command!', epheremal: true });
    }
}
});

// Log in to Discord with your client's token
client.login(token);
